@@ -31,12 +31,17 @@
 #include <clutter/clutter.h>
 #include <clutter-gst/clutter-gst.h>
 #include <gst/gst.h>
+#include <gst/basecamerabinsrc/gstcamerabin-enum.h>
+#include <gst/pbutils/encoding-profile.h>
 #include <X11/Xlib.h>
 
 #include "cheese-camera.h"
 #include "cheese-camera-device.h"
 #include "cheese-camera-device-monitor.h"
 
+#define CHEESE_VIDEO_ENC_PRESET "Profile Realtime"
+#define CHEESE_VIDEO_ENC_ALT_PRESET "Cheese Realtime"
+
 /**
  * SECTION:cheese-camera
  * @short_description: A representation of the video capture device inside
@@ -54,12 +59,6 @@ G_DEFINE_TYPE (CheeseCamera, cheese_camera, G_TYPE_OBJECT)
 
 #define CHEESE_CAMERA_ERROR cheese_camera_error_quark ()
 
-typedef enum
-{
-  MODE_IMAGE = 0,
-  MODE_VIDEO
-} GstCameraBinMode;
-
 typedef enum {
   GST_CAMERABIN_FLAG_SOURCE_RESIZE               = (1 << 0),
   GST_CAMERABIN_FLAG_SOURCE_COLOR_CONVERSION     = (1 << 1),
@@ -81,6 +80,7 @@ struct _CheeseCameraPrivate
   GstElement *effects_preview_bin;
 
   GstElement *video_source;
+  GstElement *camera_source;
   GstElement *video_file_sink;
   GstElement *audio_source;
   GstElement *audio_enc;
@@ -95,8 +95,6 @@ struct _CheeseCameraPrivate
 
   GstCaps *preview_caps;
 
-  gulong photo_handler_signal_id;
-
   gboolean is_recording;
   gboolean pipeline_is_playing;
   gchar *photo_filename;
@@ -152,36 +150,41 @@ cheese_camera_error_quark (void)
 /*
  * cheese_camera_photo_data:
  * @camera: a #CheeseCamera
- * @buffer: the #GstBuffer containing photo data
+ * @sample: the #GstSample containing photo data
  *
  * Create a #GdkPixbuf containing photo data captured from @camera, and emit it
  * in the ::photo-taken signal.
  */
 static void
-cheese_camera_photo_data (CheeseCamera *camera, GstBuffer *buffer)
+cheese_camera_photo_data (CheeseCamera *camera, GstSample *sample)
 {
+  GstBuffer          *buffer;
   GstCaps            *caps;
   const GstStructure *structure;
   gint                width, height, stride;
   GdkPixbuf          *pixbuf;
   const gint          bits_per_pixel = 8;
   guchar             *data = NULL;
+  
   CheeseCameraPrivate *priv  = camera->priv;
+  GstMapInfo         mapinfo = {0, };
 
-  caps = gst_buffer_get_caps (buffer);
+  buffer = gst_sample_get_buffer (sample);
+  caps = gst_sample_get_caps (sample);
   structure = gst_caps_get_structure (caps, 0);
   gst_structure_get_int (structure, "width", &width);
   gst_structure_get_int (structure, "height", &height);
 
-  stride = buffer->size / height;
-
-  data = g_memdup (GST_BUFFER_DATA (buffer), buffer->size);
-  pixbuf = gdk_pixbuf_new_from_data (data ? data : GST_BUFFER_DATA (buffer),
+  gst_buffer_map (buffer, &mapinfo, GST_MAP_READ);
+  stride = mapinfo.size / height;
+  data = g_memdup (mapinfo.data, mapinfo.size);
+  pixbuf = gdk_pixbuf_new_from_data (data ? data : mapinfo.data,
                                      GDK_COLORSPACE_RGB,
                                      FALSE, bits_per_pixel, width, height, stride,
                                      data ? (GdkPixbufDestroyNotify) g_free : NULL, NULL);
 
-  g_object_set (G_OBJECT (priv->camerabin), "preview-caps", NULL, NULL);
+  gst_buffer_unmap (buffer, &mapinfo);
+  g_object_set (G_OBJECT (priv->camerabin), "post-previews", FALSE, NULL);
   g_signal_emit (camera, camera_signals[PHOTO_TAKEN], 0, pixbuf);
   g_object_unref (pixbuf);
 }
@@ -198,6 +201,8 @@ cheese_camera_photo_data (CheeseCamera *camera, GstBuffer *buffer)
 static void
 cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *camera)
 {
+  CheeseCameraPrivate *priv = CHEESE_CAMERA_GET_PRIVATE (camera);
+
   switch (GST_MESSAGE_TYPE (message))
   {
     case GST_MESSAGE_WARNING:
@@ -246,20 +251,20 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
     case GST_MESSAGE_ELEMENT:
     {
       const GstStructure *structure;
-      GstBuffer *buffer;
+      GstSample *sample;
       const GValue *image;
-      if (strcmp (GST_MESSAGE_SRC_NAME (message), "camerabin") == 0)
+      if (strcmp (GST_MESSAGE_SRC_NAME (message), "camera_source") == 0)
       {
         structure = gst_message_get_structure (message);
         if (strcmp (gst_structure_get_name (structure), "preview-image") == 0)
         {
-          if (gst_structure_has_field_typed (structure, "buffer", GST_TYPE_BUFFER))
+          if (gst_structure_has_field_typed (structure, "sample", GST_TYPE_SAMPLE))
           {
-            image = gst_structure_get_value (structure, "buffer");
+            image = gst_structure_get_value (structure, "sample");
             if (image)
             {
-              buffer = gst_value_get_buffer (image);
-              cheese_camera_photo_data (camera, buffer);
+              sample = gst_value_get_sample (image);
+              cheese_camera_photo_data (camera, sample);
             }
             else
             {
@@ -268,6 +273,24 @@ cheese_camera_bus_message_cb (GstBus *bus, GstMessage *message, CheeseCamera *ca
           }
         }
       }
+      if (strcmp (GST_MESSAGE_SRC_NAME (message), "camerabin") == 0)
+      {
+        structure = gst_message_get_structure (message);
+        if (strcmp (gst_structure_get_name (structure), "image-done") == 0)
+        {
+          const gchar *filename = gst_structure_get_string (structure, "filename");
+          if (priv->photo_filename != NULL && filename != NULL &&
+              (strcmp (priv->photo_filename, filename) == 0))
+          {
+            g_signal_emit (camera, camera_signals[PHOTO_SAVED], 0);
+          }
+        }
+        else if (strcmp (gst_structure_get_name (structure), "video-done") == 0)
+        {
+          g_signal_emit (camera, camera_signals[VIDEO_SAVED], 0);
+          priv->is_recording = FALSE;
+        }
+      }
       break;
     }
     default:
@@ -401,7 +424,7 @@ cheese_camera_set_camera_source (CheeseCamera *camera)
   priv->video_source = gst_parse_bin_from_description (camera_input, TRUE, &err);
   g_free (camera_input);
 
-  if (priv->video_source == NULL)
+  if (priv->video_source == NULL || priv->camera_source == NULL)
   {
     if (err != NULL)
     {
@@ -410,7 +433,7 @@ cheese_camera_set_camera_source (CheeseCamera *camera)
     }
     return FALSE;
   }
-  g_object_set (priv->camerabin, "video-source", priv->video_source, NULL);
+  g_object_set (priv->camera_source, "video-source", priv->video_source, NULL);
 
   return TRUE;
 }
@@ -435,36 +458,63 @@ cheese_camera_set_error_element_not_found (GError **error, const gchar *factoryn
  * @camera: a #CheeseCamera
  * @error: a return location for errors, or %NULL
  *
- * Sets up the video encoder explicitly, in order to be able to control
- * properties such as encoding quality.
  */
 static void
 cheese_camera_set_video_recording (CheeseCamera *camera, GError **error)
 {
-  CheeseCameraPrivate *priv = camera->priv;
-  GstElement          *video_enc;
-  GstElement          *mux;
+  CheeseCameraPrivate *priv   = CHEESE_CAMERA_GET_PRIVATE (camera);
+  GstEncodingContainerProfile *prof;
+  GstEncodingVideoProfile *v_prof;
+  GstCaps *caps;
+  GstElement *video_enc;
+  const gchar *video_preset;
+  gboolean res;
 
-  g_return_if_fail (error == NULL || *error == NULL);
+  /* Check if we can use global preset for vp8enc. */
+  video_enc = gst_element_factory_make ("vp8enc", "vp8enc");
+  video_preset = (gchar *) &CHEESE_VIDEO_ENC_PRESET;
+  res = gst_preset_load_preset (GST_PRESET (video_enc), video_preset);
+  if (res == FALSE) {
+    g_warning("Can't find vp8enc preset: \"%s\", using alternate preset:"
+        " \"%s\". If you see this, make a bug report!",
+        video_preset, CHEESE_VIDEO_ENC_ALT_PRESET);
 
-  if ((video_enc = gst_element_factory_make ("vp8enc", "vp8enc")) == NULL)
-  {
-    cheese_camera_set_error_element_not_found (error, "vp8enc");
-    return;
-  }
-  g_object_set (priv->camerabin, "video-encoder", video_enc, NULL);
-  /* Use a preset, or set some reasonable defaults. */
-  if (!gst_preset_load_preset (GST_PRESET (video_enc), "Profile Realtime"))
-  {
-    g_object_set (G_OBJECT (video_enc), "max-latency", 1, "speed", 2, NULL);
-  }
+    /* If global preset not found, then probably we use wrong preset name,
+     * or old gstreamer version. In any case, we should try to control
+     * keep poker face and not fail. DON'T FORGET TO MAKE A BUG REPORT!*/
+    video_preset = (gchar *) &CHEESE_VIDEO_ENC_ALT_PRESET;
+    res = gst_preset_load_preset (GST_PRESET (video_enc), video_preset);
+    if (res == FALSE) {
+      g_warning ("Can't find vp8enc preset: \"%s\", "
+          "creating new userspace preset.", video_preset);
 
-  if ((mux = gst_element_factory_make ("webmmux", "webmmux")) == NULL)
-  {
-    cheese_camera_set_error_element_not_found (error, "webmmux");
-    return;
+      /* Seems like we do first run and userspace preset do not exist.
+       * Let us create a new one. It will be probably located some where here:
+       * ~/.local/share/gstreamer-1.0/presets/GstVP8Enc.prs */
+      g_object_set (G_OBJECT (video_enc), "speed", 2, NULL);
+      g_object_set (G_OBJECT (video_enc), "max-latency", 1, NULL);
+      gst_preset_save_preset (GST_PRESET (video_enc), video_preset);
+    }
   }
-  g_object_set (priv->camerabin, "video-muxer", mux, NULL);
+  gst_object_unref(video_enc);
+
+  /* create profile for webm encoding */
+  caps = gst_caps_from_string("video/webm");
+  prof = gst_encoding_container_profile_new("WebM audio/video",
+      "Standard WebM/VP8/Vorbis",
+      caps, NULL);
+
+  caps = gst_caps_from_string("video/x-vp8");
+  v_prof = gst_encoding_video_profile_new(caps, NULL, NULL, 0);
+  gst_encoding_video_profile_set_variableframerate(v_prof, TRUE);
+  gst_encoding_profile_set_preset((GstEncodingProfile*) v_prof, video_preset);
+  gst_encoding_container_profile_add_profile(prof, (GstEncodingProfile*) v_prof);
+
+  caps = gst_caps_from_string("audio/x-vorbis");
+  gst_encoding_container_profile_add_profile(prof,
+      (GstEncodingProfile*) gst_encoding_audio_profile_new(caps, NULL, NULL, 0));
+
+  g_object_set (priv->camerabin, "video-profile", prof, NULL);
 }
 
 /*
@@ -558,9 +608,9 @@ cheese_camera_create_video_filter_bin (CheeseCamera *camera, GError **error)
     cheese_camera_set_error_element_not_found (error, "videobalance");
     return FALSE;
   }
-  if ((priv->csp_post_balance = gst_element_factory_make ("ffmpegcolorspace", "csp_post_balance")) == NULL)
+  if ((priv->csp_post_balance = gst_element_factory_make ("videoconvert", "csp_post_balance")) == NULL)
   {
-    cheese_camera_set_error_element_not_found (error, "ffmpegcolorspace");
+    cheese_camera_set_error_element_not_found (error, "videoconvert");
     return FALSE;
   }
 
@@ -575,7 +625,7 @@ cheese_camera_create_video_filter_bin (CheeseCamera *camera, GError **error)
   ok &= gst_element_link_many (priv->camera_tee, priv->main_valve,
                                priv->effect_filter, priv->video_balance,
                                priv->csp_post_balance, NULL);
-  gst_pad_link (gst_element_get_request_pad (priv->camera_tee, "src%d"),
+  gst_pad_link (gst_element_get_request_pad (priv->camera_tee, "src_%u"),
                 gst_element_get_static_pad (priv->effects_preview_bin, "sink"));
 
   /* add ghostpads */
@@ -687,8 +737,9 @@ cheese_camera_switch_camera_device (CheeseCamera *camera)
  * Set the state of the GStreamer pipeline associated with the #CheeseCamera to
  * playing.
  */
-void
-cheese_camera_play (CheeseCamera *camera)
+
+static void
+cheese_camera_set_new_caps (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv;
   CheeseCameraDevice *device;
@@ -711,12 +762,17 @@ cheese_camera_play (CheeseCamera *camera)
 
   if (!gst_caps_is_empty (caps))
   {
-    g_signal_emit_by_name (priv->camerabin, "set-video-resolution-fps",
-                           priv->current_format->width,
-                           priv->current_format->height, 0, 1, 0);
+    GST_INFO_OBJECT (camera, "SETTING caps%" GST_PTR_FORMAT, caps);
+    g_object_set (priv->camerabin, "viewfinder-caps", caps, NULL);
   }
   gst_caps_unref (caps);
+}
 
+void
+cheese_camera_play (CheeseCamera *camera)
+{
+  CheeseCameraPrivate *priv   = CHEESE_CAMERA_GET_PRIVATE (camera);
+  cheese_camera_set_new_caps (camera);
   gst_element_set_state (priv->camerabin, GST_STATE_PLAYING);
   priv->pipeline_is_playing = TRUE;
 }
@@ -806,9 +862,9 @@ cheese_camera_element_from_effect (CheeseCamera *camera, CheeseEffect *effect)
                 "pipeline-desc", &effect_desc,
                 "name", &name, NULL);
 
-  effects_pipeline_desc = g_strconcat ("ffmpegcolorspace name=colorspace1 ! ",
+  effects_pipeline_desc = g_strconcat ("videoconvert name=colorspace1 ! ",
                                        effect_desc,
-                                       " ! ffmpegcolorspace name=colorspace2",
+                                       " ! videoconvert name=colorspace2",
                                        NULL);
   g_free (effect_desc);
   effect_filter = gst_parse_bin_from_description (effects_pipeline_desc, FALSE, &err);
@@ -959,7 +1015,7 @@ cheese_camera_set_tags (CheeseCamera *camera)
 
   datetime = gst_date_time_new_now_local_time();
 
-  taglist = gst_tag_list_new_full (
+  taglist = gst_tag_list_new (
       GST_TAG_APPLICATION_NAME, PACKAGE_STRING,
       GST_TAG_DATE_TIME, datetime,
       GST_TAG_DEVICE_MODEL, device_name,
@@ -990,11 +1046,9 @@ cheese_camera_start_video_recording (CheeseCamera *camera, const gchar *filename
   priv = camera->priv;
 
   g_object_set (priv->camerabin, "mode", MODE_VIDEO, NULL);
-  gst_element_set_state (priv->camerabin, GST_STATE_READY);
-  g_object_set (priv->camerabin, "filename", filename, NULL);
+  g_object_set (priv->camerabin, "location", filename, NULL);
   cheese_camera_set_tags (camera);
-  gst_element_set_state (priv->camerabin, GST_STATE_PLAYING);
-  g_signal_emit_by_name (priv->camerabin, "capture-start", 0);
+  g_signal_emit_by_name (priv->camerabin, "start-capture", 0);
   priv->is_recording = TRUE;
 }
 
@@ -1018,7 +1072,6 @@ cheese_camera_force_stop_video_recording (gpointer data)
     g_signal_emit (camera, camera_signals[VIDEO_SAVED], 0);
 
     cheese_camera_stop (camera);
-    g_object_set (priv->camerabin, "mode", MODE_IMAGE, NULL);
     cheese_camera_play (camera);
     priv->is_recording = FALSE;
   }
@@ -1046,10 +1099,7 @@ cheese_camera_stop_video_recording (CheeseCamera *camera)
 
   if (state == GST_STATE_PLAYING)
   {
-    g_signal_emit_by_name (priv->camerabin, "capture-stop", 0);
-    g_object_set (priv->camerabin, "mode", MODE_IMAGE, NULL);
-    g_signal_emit (camera, camera_signals[VIDEO_SAVED], 0);
-    priv->is_recording = FALSE;
+    g_signal_emit_by_name (priv->camerabin, "stop-capture", 0);
   }
   else
   {
@@ -1057,26 +1107,6 @@ cheese_camera_stop_video_recording (CheeseCamera *camera)
   }
 }
 
-/*
- * cheese_camera_image_done_cb:
- * @camerabin: a CameraBin #GstElement
- * @filename: (type filename): path to where the photo was saved
- * @camera: a #CheeseCamera
- *
- * Emits the ::photo-saved signal, if a filename was provided.
- */
-static void
-cheese_camera_image_done_cb (GstElement *camerabin, const gchar *filename,
-                             CheeseCamera *camera)
-{
-  CheeseCameraPrivate *priv = camera->priv;
-  g_signal_handler_disconnect (G_OBJECT (priv->camerabin),
-                               priv->photo_handler_signal_id);
-  priv->photo_handler_signal_id = 0;
-  if (priv->photo_filename != NULL)
-    g_signal_emit (camera, camera_signals[PHOTO_SAVED], 0);
-}
-
 /**
  * cheese_camera_take_photo:
  * @camera: a #CheeseCamera
@@ -1090,20 +1120,18 @@ gboolean
 cheese_camera_take_photo (CheeseCamera *camera, const gchar *filename)
 {
   CheeseCameraPrivate *priv;
+  gboolean             ready;
 
   g_return_val_if_fail (CHEESE_IS_CAMERA (camera), FALSE);
 
   priv = camera->priv;
 
-  if (priv->photo_handler_signal_id != 0)
+  g_object_get (priv->camera_source, "ready-for-capture", &ready, NULL);
+  if (!ready)
   {
     GST_WARNING ("Still waiting for previous photo data, ignoring new request");
     return FALSE;
   }
-  priv->photo_handler_signal_id = g_signal_connect (G_OBJECT (priv->camerabin),
-                                                    "image-done",
-                                                    G_CALLBACK (cheese_camera_image_done_cb),
-                                                    camera);
 
   g_free (priv->photo_filename);
   priv->photo_filename = g_strdup (filename);
@@ -1112,22 +1140,13 @@ cheese_camera_take_photo (CheeseCamera *camera, const gchar *filename)
 
   /* Only copy the data if we're giving away a pixbuf,
    * not if we're throwing everything away straight away */
-
-  if (priv->photo_filename != NULL)
-  {
-    g_object_set (priv->camerabin, "filename", priv->photo_filename, NULL);
-    g_object_set (priv->camerabin, "mode", MODE_IMAGE, NULL);
-    cheese_camera_set_tags (camera);
-    g_signal_emit_by_name (priv->camerabin, "capture-start", 0);
-  }
-  else
-  {
-    g_signal_handler_disconnect (G_OBJECT (priv->camerabin),
-                                 priv->photo_handler_signal_id);
-    priv->photo_handler_signal_id = 0;
+  if (priv->photo_filename == NULL)
     return FALSE;
-  }
 
+  g_object_set (priv->camerabin, "location", priv->photo_filename, NULL);
+  g_object_set (priv->camerabin, "mode", MODE_IMAGE, NULL);
+  cheese_camera_set_tags (camera);
+  g_signal_emit_by_name (priv->camerabin, "start-capture", 0);
   return TRUE;
 }
 
@@ -1145,24 +1164,23 @@ cheese_camera_take_photo_pixbuf (CheeseCamera *camera)
 {
   CheeseCameraPrivate *priv;
   GstCaps             *caps;
+  gboolean             ready;
 
   g_return_val_if_fail (CHEESE_IS_CAMERA (camera), FALSE);
 
   priv = camera->priv;
 
-  if (priv->photo_handler_signal_id != 0)
+  g_object_get (priv->camera_source, "ready-for-capture", &ready, NULL);
+  if (!ready)
   {
     GST_WARNING ("Still waiting for previous photo data, ignoring new request");
     return FALSE;
   }
-  priv->photo_handler_signal_id = g_signal_connect (G_OBJECT (priv->camerabin),
-                                                    "image-done",
-                                                    G_CALLBACK (cheese_camera_image_done_cb),
-                                                    camera);
-  caps = gst_caps_new_simple ("video/x-raw-rgb",
-                              "bpp", G_TYPE_INT, 24,
-                              "depth", G_TYPE_INT, 24,
+
+  caps = gst_caps_new_simple ("video/x-raw",
+                              "format", G_TYPE_STRING, "RGB",
                               NULL);
+  g_object_set (G_OBJECT (priv->camerabin), "post-previews", TRUE, NULL);
   g_object_set (G_OBJECT (priv->camerabin), "preview-caps", caps, NULL);
   gst_caps_unref (caps);
 
@@ -1172,9 +1190,9 @@ cheese_camera_take_photo_pixbuf (CheeseCamera *camera)
 
   /* Take the photo */
 
-  g_object_set (priv->camerabin, "filename", "/dev/null", NULL);
+  g_object_set (priv->camerabin, "location", NULL, NULL);
   g_object_set (priv->camerabin, "mode", MODE_IMAGE, NULL);
-  g_signal_emit_by_name (priv->camerabin, "capture-start", 0);
+  g_signal_emit_by_name (priv->camerabin, "start-capture", 0);
 
   return TRUE;
 }
@@ -1398,9 +1416,10 @@ cheese_camera_init (CheeseCamera *camera)
   priv->photo_filename          = NULL;
   priv->camera_devices          = NULL;
   priv->device_node             = NULL;
-  priv->photo_handler_signal_id = 0;
   priv->current_format          = NULL;
   priv->monitor                 = NULL;
+  priv->camera_source           = NULL;
+  priv->video_source            = NULL;
 }
 
 /**
@@ -1495,7 +1514,6 @@ cheese_camera_setup (CheeseCamera *camera, const gchar *uuid, GError **error)
   CheeseCameraPrivate *priv;
   GError  *tmp_error = NULL;
   GstElement *video_sink;
-  GstCaps *caps;
 
   g_return_if_fail (error == NULL || *error == NULL);
   g_return_if_fail (CHEESE_IS_CAMERA (camera));
@@ -1519,10 +1537,12 @@ cheese_camera_setup (CheeseCamera *camera, const gchar *uuid, GError **error)
   if ((priv->camerabin = gst_element_factory_make ("camerabin", "camerabin")) == NULL)
   {
     cheese_camera_set_error_element_not_found (error, "camerabin");
-    return;
   }
-  g_object_set (priv->camerabin, "video-capture-height", 0,
-                "video-capture-width", 0, NULL);
+  if ((priv->camera_source = gst_element_factory_make ("wrappercamerabinsrc", "camera_source")) == NULL)
+  {
+    cheese_camera_set_error_element_not_found (error, "wrappercamerabinsrc");
+  }
+  g_object_set (priv->camerabin, "camera-source", priv->camera_source, NULL);
 
   /* Create a clutter-gst sink and set it as camerabin sink*/
 
@@ -1532,12 +1552,10 @@ cheese_camera_setup (CheeseCamera *camera, const gchar *uuid, GError **error)
     return;
   }
   g_object_set (G_OBJECT (video_sink), "async", FALSE, NULL);
+  g_object_set (G_OBJECT (video_sink), "sync", FALSE, NULL);
   g_object_set (G_OBJECT (priv->camerabin), "viewfinder-sink", video_sink, NULL);
 
-  /* Set caps to filter, so it doesn't defaults to I420 format*/
-  caps = gst_caps_from_string ("video/x-raw-yuv; video/x-raw-rgb");
-  g_object_set (G_OBJECT (priv->camerabin), "filter-caps", caps, NULL);
-  gst_caps_unref (caps);
+  /* Set flags to enable conversions*/
 
   cheese_camera_set_camera_source (camera);
   cheese_camera_set_video_recording (camera, &tmp_error);
@@ -1550,7 +1568,8 @@ cheese_camera_setup (CheeseCamera *camera, const gchar *uuid, GError **error)
     GST_WARNING ("%s", (*error)->message);
     return;
   }
-  g_object_set (G_OBJECT (priv->camerabin), "video-source-filter", priv->video_filter_bin, NULL);
+
+  g_object_set (G_OBJECT (priv->camera_source), "video-source-filter", priv->video_filter_bin, NULL);
 
   priv->bus = gst_element_get_bus (priv->camerabin);
   gst_bus_add_signal_watch (priv->bus);
@@ -1747,42 +1766,43 @@ cheese_camera_set_balance_property (CheeseCamera *camera, const gchar *property,
 gchar *
 cheese_camera_get_recorded_time (CheeseCamera *camera)
 {
-  g_return_val_if_fail (CHEESE_IS_CAMERA (camera), NULL);
-
   CheeseCameraPrivate *priv = camera->priv;
+  GstFormat format = GST_FORMAT_TIME;
+  gint64 curtime;
+  GstElement *videosink;
   const gint TUNIT_60 = 60;
-  GstClock *clock;
-  GstClockTime base_time;
-  GstClockTime clock_time;
-  GstClockTimeDiff time_diff;
   gint total_time;
   gint hours;
   gint minutes;
   gint seconds;
+  gboolean ret = FALSE;
 
-  base_time  = gst_element_get_base_time (priv->camerabin);
-  clock = gst_element_get_clock (priv->camerabin);
-  clock_time = gst_clock_get_time (clock);
-  gst_object_unref (clock);
-  /* In order to calculate the running time of the stream,
-   * it is necessary to substract the base_time from the
-   * clock_time. The result is expressed in nanoseconds.
-   * */
-  time_diff = GST_CLOCK_DIFF (base_time, clock_time);
+  g_return_val_if_fail (CHEESE_IS_CAMERA (camera), NULL);
 
-  // Substract seconds, minutes and hours.
-  total_time = GST_TIME_AS_SECONDS (time_diff);
-  seconds = total_time % TUNIT_60;
-  total_time = total_time - seconds;
-  minutes = (total_time % (TUNIT_60 * TUNIT_60)) / TUNIT_60;
-  total_time = total_time - (minutes * TUNIT_60);
-  hours = total_time / (TUNIT_60 * TUNIT_60);
+  videosink = gst_bin_get_by_name (GST_BIN_CAST (priv->camerabin), "videobin-filesink");
+  if (videosink) {
+    ret = gst_element_query_position (videosink, format, &curtime);
+    gst_object_unref (videosink);
+  }
+  if (ret) {
 
-  /* Translators: This is a time format, like "09:05:02" for 9
-   * hours, 5 minutes, and 2 seconds. You may change ":" to
-   * the separator that your locale uses or use "%Id" instead
-   * of "%d" if your locale uses localized digits.
-   */
-  return g_strdup_printf (C_("time format", "%02i:%02i:%02i"),
-                          hours, minutes, seconds);
+    // Substract seconds, minutes and hours.
+    total_time = GST_TIME_AS_SECONDS (curtime);
+    seconds = total_time % TUNIT_60;
+    total_time = total_time - seconds;
+    minutes = (total_time % (TUNIT_60 * TUNIT_60)) / TUNIT_60;
+    total_time = total_time - (minutes * TUNIT_60);
+    hours = total_time / (TUNIT_60 * TUNIT_60);
+
+    /* Translators: This is a time format, like "09:05:02" for 9
+     * hours, 5 minutes, and 2 seconds. You may change ":" to
+     * the separator that your locale uses or use "%Id" instead
+     * of "%d" if your locale uses localized digits.
+     */
+    return g_strdup_printf (C_("time format", "%02i:%02i:%02i"),
+                            hours, minutes, seconds);
+  } else {
+    GST_WARNING ("Failed to get time from video filesink from camerabin");
+    return "";
+  }
 }
